"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildFragments = void 0;
const _models_1 = require("../models");
const _generators_1 = require("../generators");
const _store_1 = require("../store");
const TYPES_TO_PARSE = [_models_1.TypeKind.OBJECT, _models_1.TypeKind.INTERFACE];
const forbiddenTypes = [_models_1.MethodType.Query, _models_1.MethodType.Mutation, _models_1.MethodType.Subscription];
function buildFragments() {
    const { schemaTypes } = _store_1.SchemaStore;
    return schemaTypes
        .map((type) => {
        if (TYPES_TO_PARSE.includes(type.kind) && !forbiddenTypes.includes(type.name)) {
            const isConnection = _store_1.SchemaStore.isTypeConnection(type.name);
            const fragment = _generators_1.generateNormalFragment(type, isConnection);
            if (fragment) {
                return `export const ${type.name}Fragment = gql\` 
  fragment ${type.name}Fragment on ${type.name} {
    ${fragment}
  }
\`;`;
            }
        }
    })
        .filter((value) => !!value);
}
exports.buildFragments = buildFragments;
