"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.saveFile = void 0;
const ora_1 = __importDefault(require("ora"));
const prettier = __importStar(require("prettier"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const ts = __importStar(require("typescript"));
const chalk_1 = __importDefault(require("chalk"));
const mkdirp_1 = __importDefault(require("mkdirp"));
let saveModels;
async function saveFile(template, output, jsMode) {
    try {
        const pretty = ora_1.default('\nRunning Prettier with your config on the generated output').start();
        let prettierFoundOptions = await prettier.resolveConfig(process.cwd());
        let message = 'using your Prettier config';
        if (!prettierFoundOptions) {
            message = 'using default Prettier config';
            prettierFoundOptions = {
                printWidth: 100,
                tabWidth: 2,
                trailingComma: 'es5',
                singleQuote: true,
                semi: true,
                bracketSpacing: true,
                htmlWhitespaceSensitivity: 'strict',
            };
        }
        const formatedModelsFile = prettier.format(template, {
            ...prettierFoundOptions,
            parser: 'typescript',
        });
        pretty.succeed(`💄 Your file has been formated ${message}`);
        if (output) {
            const outputfile = path_1.default.resolve(process.cwd(), output);
            if (fs_1.default.existsSync(outputfile)) {
                const content = await writeOutput(outputfile, formatedModelsFile, jsMode);
                return content;
            }
            else {
                let dirList = outputfile.split('/');
                dirList.pop();
                const dirPath = dirList.join('/');
                await mkdirp_1.default(dirPath);
                return await writeOutput(outputfile, formatedModelsFile, jsMode);
            }
        }
        return formatedModelsFile;
    }
    catch (e) {
        return Promise.reject(e);
    }
}
exports.saveFile = saveFile;
function TypescriptCompile(fileNames, options) {
    let program = ts.createProgram(fileNames, options);
    program.emit();
}
async function writeOutput(path, content, jsMode) {
    try {
        saveModels = ora_1.default('Saving models file...').start();
        await fs_1.default.writeFileSync(path, content);
        saveModels.succeed(`🎉 Output saved at ${chalk_1.default.blue(chalk_1.default.bold(`${path}`))}`);
        if (jsMode) {
            try {
                TypescriptCompile([path], {
                    module: ts.ModuleKind.CommonJS,
                    target: ts.ScriptTarget.ES2018,
                    moduleResolution: ts.ModuleResolutionKind.NodeJs,
                    allowSyntheticDefaultImports: true,
                    experimentalDecorators: true,
                    emitDecoratorMetadata: true,
                    resolveJsonModule: true,
                    esModuleInterop: true,
                    removeComments: true,
                    noImplicitAny: false,
                    noUnusedLocals: false,
                    pretty: true,
                    sourceMap: false,
                    downlevelIteration: true,
                    declaration: true,
                    skipLibCheck: true,
                    types: ['node'],
                });
                fs_1.default.unlinkSync(path);
                return content;
            }
            catch (e) {
                return Promise.reject(e);
            }
        }
        else {
            return content;
        }
    }
    catch (e) {
        saveModels.fail('Saving models file failed');
        console.log(e.message);
        return Promise.reject('Error in saving file');
    }
}
