"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.retrieveIntrospectionSchema = exports.downloadSchema = void 0;
const chalk_1 = __importDefault(require("chalk"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const ora_1 = __importDefault(require("ora"));
const query = __importStar(require("querystringify"));
const graphql_1 = require("graphql");
const buildClientSchema_1 = require("graphql/utilities/buildClientSchema");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const possibleGraphQLSuffix = ['/graphql', '/api', '/__graphql', '/__api'];
async function downloadSchema(endpoint, header) {
    const download = ora_1.default(`⬇️  Downloading schemas from ${chalk_1.default.blue(endpoint)}`).start();
    try {
        let formatedHeaders = getHeadersFromInput(header);
        formatedHeaders = {
            ...formatedHeaders,
            'Content-Type': 'application/json',
        };
        const schema = await getRemoteSchema(endpoint, {
            method: 'POST',
            json: true,
            headers: formatedHeaders,
        });
        if (schema.status === 'err') {
            download.fail();
            return Promise.reject(schema.message);
        }
        else {
            download.succeed(`📥 Schemas successfully downloaded from ${chalk_1.default.blue(endpoint)}`);
            return schema.schema;
        }
    }
    catch (e) {
        download.fail();
        return Promise.reject(`❗️ Unable to request from the GraphQL Api, please verify that the server is online \n ${chalk_1.default.yellow(`Maybe the endpoint is not a GraphQL Api, try to add ${chalk_1.default.green(possibleGraphQLSuffix.join(', '))} at the end of your url`)}
      `);
    }
}
exports.downloadSchema = downloadSchema;
function getHeadersFromInput(header) {
    switch (typeof header) {
        case 'string': {
            const keys = query.parse(header);
            const key = Object.keys(keys)[0];
            return { [key]: keys[key] };
        }
        case 'object': {
            return header;
        }
        default: {
            return {};
        }
    }
}
async function getRemoteSchema(endpoint, options) {
    try {
        const { data, errors } = await node_fetch_1.default(endpoint, {
            method: options.method,
            headers: options.headers,
            body: JSON.stringify({ query: graphql_1.getIntrospectionQuery() }),
        }).then((res) => res.json());
        if (errors) {
            return { status: 'err', message: JSON.stringify(errors, null, 2) };
        }
        if (options.json) {
            return {
                status: 'ok',
                schema: JSON.stringify(data, null, 2),
            };
        }
        else {
            const schema = buildClientSchema_1.buildClientSchema(data);
            return {
                status: 'ok',
                schema: graphql_1.printSchema(schema),
            };
        }
    }
    catch (err) {
        return Promise.reject(err);
    }
}
async function retrieveIntrospectionSchema({ endpoint, header, json, download, }) {
    try {
        if (endpoint) {
            const JSONschema = await downloadSchema(endpoint, header);
            if (download) {
                const outputfile = path_1.default.resolve(process.cwd(), download);
                try {
                    await fs_1.default.writeFileSync(outputfile, JSONschema);
                    console.log(chalk_1.default.green(`Graphql intropesction schema saved at ${download}`));
                }
                catch (e) {
                    console.error(e);
                }
            }
            return JSON.parse(JSONschema);
        }
        else if (json) {
            return require(path_1.default.resolve(process.cwd(), json));
        }
        else {
            return null;
        }
    }
    catch (e) {
        return Promise.reject(e);
    }
}
exports.retrieveIntrospectionSchema = retrieveIntrospectionSchema;
